/*
 * PlaybackController.h
 *
 *  Created on: 09.07.2010
 *      Author: darkstar
 */

#ifndef PLAYBACKCONTROLLER_H_
#define PLAYBACKCONTROLLER_H_

#include <qobject.h>

#include "media.h"
#include "playlist.h"
#include "playlistaspectlistviewrenderer.h"

class MPlayer;
class QTimer;

class PlaybackController : public QObject
{
	Q_OBJECT
public:
	enum PlayOrder {
		ORDER_NORMAL = 0,
		ORDER_REPEAT_ALL = 1,
		ORDER_REPEAT_ONE = 2,
		ORDER_RANDOM = 3,
		ORDER_REPEAT_RANDOM = 4,
		ORDER_MAX = 5
	};

	PlaybackController();
	virtual ~PlaybackController();

	void attachTo(PlayListAspectListViewRenderer *source);
	void detach();
	PlayListAspectListViewRenderer *aspectSource() { return aspectSource_; }

	void setPlayOrder(PlayOrder);

	// index based methods:
	int activeItemIndex();
	void setActiveItemIndex(int itemIndex);

	int nextActiveItemIndex(bool modifyRandomPlayOrderTableIndex = false);
	int previousActiveItemIndex(bool modifyRandomPlayOrderTableIndex = false);

	// item based methods:
	PlayListItem *activeItem();
	void setActiveItem(PlayListItem *item);

	PlayListItem *nextItem(bool modifyRandomPlayOrderTableIndex = false);
	PlayListItem *previousItem(bool modifyRandomPlayOrderTableIndex = false);

	bool isPlaying();

public slots:
	void stopCurrentPlaybackAndPlayCurrentItem();

	void startOrPauseCurrentPlaybackOrStartNewPlayback();
	void stopCurrentPlayback();
	void playNextItem();
	void playPreviousItem();

	void setAbsoluteSeconds(int secs);
	void setRelativeSeconds(int secs);

signals:
	void activeItemChanged();

	void startingPlayback();
	void playbackStarted();

	void stoppingPlayback(bool temporarily);
	void playbackStopped(bool temporarily);

	void playbackUnpaused();
	void playbackPaused();

	void currentPlayTime(int secs);
	void mediaPrefetched(PlayListItem *item);

	void playbackSeekedToAbsoluteSeconds(int secs);

	void problemWithMedia(PlayListItem *);
	void playerInitializationFailed(PlayListItem *);

private slots:
	void updateView();
	void osdTimerTimedUpdate();
	void mplayerErrorMessage(QString);
	void mplayerCurrentPlayTime(int seconds);

private:
	PlayListAspectListViewRenderer *aspectSource_;

	PlayOrder playOrder_;
	int *randomPlayOrderTable;

	int count_;
	int randomPlayOrderTableIndex_;
	int activeItemIndex_;
	PlayListItem activeItem_;

	QTimer *osdTimer_;
	MPlayer *mplayer_;
	int keyEventCount_;
	bool nextMediaPrefetched_;
	int currentMediaLength_;

	int getSelectedItemIndex();

	void updateRandomPlayOrderTable();
	void generateRandomPlayOrderTable();

	void startOrPauseCurrentPlaybackOrStartNewPlayback(bool forcePlayNewFile);

	void internalStartPlayback();
	void internalStopPlayback(bool temporarily = false, bool stopBackend = true);
};

#endif /* PLAYBACKCONTROLLER_H_ */
